#include "mod_jbm.h"

/************************
 * Forward Declarations *
 ************************/
extern "C" module MODULE_VAR_EXPORT jbm_module;

/**************************
 * Configuration Handlers *
 **************************/

/* per directory configuration structure. */
typedef struct {
    char *m_host;
    char *m_user;
    char *m_pass;
    char *m_dbname;
    int   m_port;
} jbm_dir_config;

static void *jbm_create_dir_config(pool *p, char *path)
{
    jbm_dir_config *cfg = 
        (jbm_dir_config *)ap_pcalloc(p, sizeof(jbm_dir_config));
    cfg->m_host     = "localhost";
    cfg->m_user     = "root";
    cfg->m_pass     = "";
    cfg->m_dbname   = "jbm";
    cfg->m_port     = 3306;
    return (void *)cfg;
}

typedef const char *(*CMD_HAND_TYPE) ();
static const command_rec jbm_cmds[] =
{
    {
        "JBMMysqlHost",                             /* directive name */
        ap_set_string_slot,          /* built in handler */
        (void *)XtOffsetOf(jbm_dir_config, m_host),
        ACCESS_CONF,                                /* where available */
        TAKE1,                                      /* arguments */
        "JBM mysql hostname"                        /* directive description */
    },
    {
        "JBMMysqlUser",                             /* directive name */
        ap_set_string_slot,          /* built in handler */
        (void *)XtOffsetOf(jbm_dir_config, m_user),
        ACCESS_CONF,                                /* where available */
        TAKE1,                                      /* arguments */
        "JBM mysql user name"                       /* directive description */
    },
    {
        "JBMMysqlPass",                             /* directive name */
        ap_set_string_slot,          /* built in handler */
        (void *)XtOffsetOf(jbm_dir_config, m_pass),
        ACCESS_CONF,                                /* where available */
        TAKE1,                                      /* arguments */
        "JBM mysql user name"                       /* directive description */
    },
    {
        "JBMMysqlDBName",                           /* directive name */
        ap_set_string_slot,          /* built in handler */
        (void *)XtOffsetOf(jbm_dir_config, m_dbname),
        ACCESS_CONF,                                /* where available */
        TAKE1,                                      /* arguments */
        "JBM mysql db name"                         /* directive description */
    },
    {
        "JBMMysqlPort",                             /* directive name */
        jbm_set_port,                               /* built in handler */
        NULL,
        ACCESS_CONF,                                /* where available */
        TAKE1,                                      /* arguments */
        "JBM mysql db port"                         /* directive description */
    },
    {NULL, NULL, NULL, 0, cmd_how(0), NULL}
};

static const char *jbm_set_port(cmd_parms *parms, void *mconfig, const char *arg)
{
    jbm_dir_config *cfg = (jbm_dir_config *)mconfig;
    if(strlen(arg) > 0)
    {
        cfg->m_port = atoi(arg);
    }
    return NULL;
}


/********************
 * Content Handlers *
 ********************/

/* The main event */
static int jbm_handler(request_rec *r)
{
    // Get the config for this request.
    jbm_dir_config *cfg = 
        (jbm_dir_config *)
        ap_get_module_config(r->per_dir_config, &jbm_module);

    // Generate the message.
    string messagetosend = 
        string("<P>") + 
        string(cfg->m_host) + 
        string("<br>\n") +
        string(cfg->m_user) + 
        string("<br>\n") +
        string(cfg->m_pass) + 
        string("<br>\n") +
        string(cfg->m_dbname) + 
        string("<br>\n") +
//        cfg->m_port + 
        string("</P>\n");

    // Send the headers.
    r->content_type = "text/html";
    ap_send_http_header(r);

    if (!r->header_only) {
        ap_rputs(messagetosend.c_str(), r);
    }

    return OK;
}

/* Dispatch list of content handlers */
static const handler_rec jbm_handlers[] = { 
    { "jbm", jbm_handler }, 
    { NULL, NULL }
};

/************************
 * Global Dispatch List *
 ************************/

// We have to use C style linkage for the API functions that will be
// linked to apache.
extern "C" {
    // Dispatch list for API hooks 
    module MODULE_VAR_EXPORT jbm_module = {
        STANDARD_MODULE_STUFF, 
        NULL,                       /* module initializer                  */
        jbm_create_dir_config,      /* create per-dir    config structures */
        NULL,                       /* merge  per-dir    config structures */
        NULL,                       /* create per-server config structures */
        NULL,                       /* merge  per-server config structures */
        jbm_cmds,                   /* table of config file commands       */
        jbm_handlers,               /* [#8] MIME-typed-dispatched handlers */
        NULL,                       /* [#1] URI to filename translation    */
        NULL,                       /* [#4] validate user id from request  */
        NULL,                       /* [#5] check if the user is ok _here_ */
        NULL,                       /* [#3] check access by host address   */
        NULL,                       /* [#6] determine MIME type            */
        NULL,                       /* [#7] pre-run fixups                 */
        NULL,                       /* [#9] log a transaction              */
        NULL,                       /* [#2] header parser                  */
        NULL,                       /* child_init                          */
        NULL,                       /* child_exit                          */
        NULL                        /* [#0] post read-request              */
#ifdef EAPI
            ,NULL,                      /* EAPI: add_module                    */
        NULL,                       /* EAPI: remove_module                 */
        NULL,                       /* EAPI: rewrite_command               */
        NULL,                       /* EAPI: new_connection                */
        NULL                        /* EAPI: close_connection              */
#endif
    };
};


