#include "BulkBuffer.h"

BulkBuffer* BulkBuffer::inst = NULL; // initialize pointer

// {{{ BulkBuffer::BulkBuffer()
BulkBuffer::BulkBuffer()
{
    data_counter = physical_counter = 0;
    add();
}
// }}}

// {{{ BulkBuffer::~BulkBuffer()
BulkBuffer::~BulkBuffer()
{
    while(buffer.size())
    {
        delete buffer[buffer.size()-1];
    }
    buffer.clear();
}
// }}}

// {{{ BulkBuffer &BulkBuffer::instance()
BulkBuffer &BulkBuffer::instance()
{
    if(inst == NULL) inst = new BulkBuffer;
    return *inst;
}
// }}}

// {{{ BulkBuffer *BulkBuffer::instancePtr()
BulkBuffer *BulkBuffer::instancePtr()
{
    if(inst == NULL) inst = new BulkBuffer;
    return inst;
}
// }}}

// {{{ void BulkBuffer::flush()
void BulkBuffer::flush()
{
    while(buffer.size() > 1)
    {
        delete (BulkBufferBlock *)buffer[buffer.size()-1];
        buffer.pop_back();
    }
    buffer[0]->flush();
    data_counter = 0; 
    physical_counter = buffer[0]->sz;
    return ;
}
// }}}

// {{{ bool BulkBuffer::set(const char *ori, char **dest)
bool BulkBuffer::set(const char *ori, char **dest)
{
    int size = strlen(ori);
    return set(ori, dest, size);
}
// }}}

// {{{ bool BulkBuffer::set(const char *ori, char **dest, const unsigned int size)
bool BulkBuffer::set(const char *ori, char **dest, const unsigned int size)
{
    if(BULKBUFFERBLOCK_SIZE <= size)
    {
        add(size+1);
        return setData(buffer.size()-1, ori, dest, size);
    }

    for(unsigned int idx=0; idx < buffer.size(); idx++)
    {
        if(buffer[idx]->sp < size+1) continue;

        return setData(idx, ori, dest, size);
    }

    add();
    return setData(buffer.size()-1, ori, dest, size);
}
// }}}

// {{{ bool BulkBuffer::space(char **dest, const unsigned int size)
bool BulkBuffer::space(char **dest, const unsigned int size)
{
    if(BULKBUFFERBLOCK_SIZE <= size)
    {
        add(size);
        return setSpace(buffer.size()-1, dest, size);
    }

    for(unsigned int idx=0; idx < buffer.size(); idx++)
    {
        if(buffer[idx]->sp < size) continue;

        return setSpace(idx, dest, size);
    }

    add();
    return setSpace(buffer.size()-1, dest, size);
}
// }}}


// {{{ bool BulkBuffer::setData(unsigned int idx, const char *ori, char **dest, const unsigned int size)
bool BulkBuffer::setData(unsigned int idx, const char *ori, char **dest, const unsigned int size)
{
    bool ret = (buffer[idx])->set(ori, dest, size);
    if(ret)
    {
        data_counter += (size+1);
        return true;
    }else
    {
        return false;
    }
}
// }}}

// {{{ bool BulkBuffer::setSpace(unsigned int idx, char **dest, const unsigned int size)
bool BulkBuffer::setSpace(unsigned int idx, char **dest, const unsigned int size)
{
    bool ret = (buffer[idx])->space(dest, size);
    if(ret)
    {
        data_counter += size;
        return true;
    }else
    {
        return false;
    }
}
// }}}

// {{{ bool BulkBuffer::add(const unsigned int size)
bool BulkBuffer::add(const unsigned int size)
{
    buffer.push_back(new BulkBufferBlock(size));
    physical_counter += size;
    return true;
}
// }}}

// {{{ bool BulkBuffer::add()
bool BulkBuffer::add()
{
    return add(BULKBUFFERBLOCK_SIZE);
}
// }}}

// {{{ unsigned int BulkBuffer::blocks()
unsigned int BulkBuffer::blocks()
{
    return buffer.size();
}
// }}}

/* 
   * Local variables: 
   * tab-width: 4 
   * c-basic-offset: 4 
   * End: 
   * vim600: noet sw=4 ts=4 fdm=marker 
   * vim<600: noet sw=4 ts=4 
   */
