#include "JbmCommon.h"

JbmCommon* JbmCommon::inst = NULL;

// {{{ JbmCommon &JbmCommon::instance()
JbmCommon &JbmCommon::instance()
{
        if(inst == NULL) inst = new JbmCommon;
            return *inst;
}
// }}}

// {{{ JbmCommon *JbmCommon::instancePtr()
JbmCommon *JbmCommon::instancePtr()
{
        if(inst == NULL) inst = new JbmCommon;
            return inst;
}
// }}}

// {{{ int JbmCommon::log(int level, char *srcfile, int line, char *fmt, ...)
int JbmCommon::log(int level, char *srcfile, int line, char *fmt, ...)
{
    va_list args;
    if ((level & logLevel) == 0)
        return 0;

    if (srcfile == NULL || fmt == NULL || *srcfile == 0 || *fmt == 0)
        return -1;

    va_start (args, fmt);
    vsprintf(_printBuf, fmt, args);
    va_end(args);

    fprintf(stdout, "%s(%d) %s", srcfile, line, _printBuf);
    fflush(stdout);

    return 0;
}
// }}}

// {{{ int JbmCommon::lfatal(char *srcfile, int line, char *fmt, ...)
int JbmCommon::lfatal(char *srcfile, int line, char *fmt, ...)
{
    va_list args;
    if ((JBM_LOG_FATAL & logLevel) == 0)
        return 0;

    if (srcfile == NULL || fmt == NULL || *srcfile == 0 || *fmt == 0)
        return -1;

    va_start (args, fmt);
    vsprintf(_printBuf, fmt, args);
    va_end(args);

    fprintf(stdout, "%s(%d) %s", srcfile, line, _printBuf);
    fflush(stdout);

    return 0;
}
// }}}

// {{{ int JbmCommon::lerror(char *srcfile, int line, char *fmt, ...)
int JbmCommon::lerror(char *srcfile, int line, char *fmt, ...)
{
    va_list args;
    if ((JBM_LOG_ERROR & logLevel) == 0)
        return 0;

    if (srcfile == NULL || fmt == NULL || *srcfile == 0 || *fmt == 0)
        return -1;

    va_start (args, fmt);
    vsprintf(_printBuf, fmt, args);
    va_end(args);

    fprintf(stdout, "%s(%d) %s", srcfile, line, _printBuf);
    fflush(stdout);

    return 0;
}
// }}}

// {{{ int JbmCommon::lwarning(char *srcfile, int line, char *fmt, ...)
int JbmCommon::lwarning(char *srcfile, int line, char *fmt, ...)
{
    va_list args;
    if ((JBM_LOG_WARNING & logLevel) == 0)
        return 0;

    if (srcfile == NULL || fmt == NULL || *srcfile == 0 || *fmt == 0)
        return -1;

    va_start (args, fmt);
    vsprintf(_printBuf, fmt, args);
    va_end(args);

    fprintf(stdout, "%s(%d) %s", srcfile, line, _printBuf);
    fflush(stdout);

    return 0;
}
// }}}

// {{{ int JbmCommon::lnotice(char *srcfile, int line, char *fmt, ...)
int JbmCommon::lnotice(char *srcfile, int line, char *fmt, ...)
{
    va_list args;
    if ((JBM_LOG_NOTICE & logLevel) == 0)
        return 0;

    if (srcfile == NULL || fmt == NULL || *srcfile == 0 || *fmt == 0)
        return -1;

    va_start (args, fmt);
    vsprintf(_printBuf, fmt, args);
    va_end(args);

    fprintf(stdout, "%s(%d) %s", srcfile, line, _printBuf);
    fflush(stdout);

    return 0;
}
// }}}

// {{{ int JbmCommon::ldebug(char *srcfile, int line, char *fmt, ...)
int JbmCommon::ldebug(char *srcfile, int line, char *fmt, ...)
{
    va_list args;
    if ((JBM_LOG_DEBUG & logLevel) == 0)
        return 0;

    if (srcfile == NULL || fmt == NULL || *srcfile == 0 || *fmt == 0)
        return -1;

    va_start (args, fmt);
    vsprintf(_printBuf, fmt, args);
    va_end(args);

    fprintf(stdout, "%s(%d) %s", srcfile, line, _printBuf);
    fflush(stdout);

    return 0;
}
// }}}

// {{{ const char *JbmCommon::md5Hash(const unsigned char *str, unsigned int len)
const char *JbmCommon::md5Hash(const unsigned char *str, unsigned int len)
{
    char *idx;
    int  i;

    md5((unsigned char *)str, len, _md5Hash);
    for(i = 0, idx =_md5Buf; i < JBM_MD5_HASH_SIZE; i++,idx+=2)
        sprintf(idx, "%02x", _md5Hash[i] );

    return (const char *)_md5Buf;
}
// }}}

// {{{ const char *JbmCommon::md5Hash(const char *str, unsigned int len)
const char *JbmCommon::md5Hash(const char *str, unsigned int len)
{
    return md5Hash((unsigned char *)str, len);
}
// }}}

// {{{ const char *JbmCommon::sha1Hash(const unsigned char *str, unsigned int len)
const char *JbmCommon::sha1Hash(const unsigned char *str, unsigned int len)
{
    char *idx;
    int  i;

    sha1((unsigned char *)str, len, _sha1Hash);
    for(i = 0, idx =_sha1Buf; i < JBM_SHA1_HASH_SIZE; i++,idx+=2)
        sprintf(idx, "%02x", _sha1Hash[i] );

    return _sha1Buf;
}
// }}}

// {{{ const char *JbmCommon::sha1Hash(const char *str, unsigned int len)
const char *JbmCommon::sha1Hash(const char *str, unsigned int len)
{
    return sha1Hash((unsigned char *)str, len);
}
// }}}


