#ifndef __JBMCOMMON_H__
#define __JBMCOMMON_H__

#include <iostream>
#include <stdio.h>
#include <string.h>
#include "md5.h"
#include "sha1.h"

extern "C"
{
#include <stdarg.h>
} // extern "C"

#define JBM_MD5_HASH_SIZE       16
#define JBM_MD5_BUF_SIZE        JBM_MD5_HASH_SIZE*2+1
#define JBM_SHA1_HASH_SIZE      20
#define JBM_SHA1_BUF_SIZE       JBM_SHA1_HASH_SIZE*2+1
#define JBM_MAX_PRINTBUF_SIZE   4096

#define JBM_LOG_FATAL           0x001
#define JBM_LOG_ERROR           0x001 << 1
#define JBM_LOG_WARNING         0x001 << 2
#define JBM_LOG_NOTICE          0x001 << 3
#define JBM_LOG_DEBUG           0x001 << 4
#define JBM_DEFAULT_LOG_LEVEL   JBM_LOG_FATAL | JBM_LOG_ERROR | JBM_LOG_WARNING | JBM_LOG_NOTICE | JBM_LOG_DEBUG 

#ifndef __int64
#define __int64 long long
#endif

#define JC      JbmCommon::instance()

class JbmCommon
{
private:
    static JbmCommon    *inst;
    char                _printBuf[JBM_MAX_PRINTBUF_SIZE];
    unsigned char       _md5Hash[JBM_MD5_HASH_SIZE];
    char                _md5Buf[JBM_MD5_BUF_SIZE];
    unsigned char       _sha1Hash[JBM_SHA1_HASH_SIZE];
    char                _sha1Buf[JBM_SHA1_BUF_SIZE];

    JbmCommon()
    : logLevel(JBM_DEFAULT_LOG_LEVEL)
    {
        memset(_printBuf, 0x00, JBM_MAX_PRINTBUF_SIZE);
        memset(_md5Hash, 0x00, JBM_MD5_HASH_SIZE);
        memset(_md5Buf, 0x00, JBM_MD5_BUF_SIZE);
        memset(_sha1Hash, 0x00, JBM_SHA1_HASH_SIZE);
        memset(_sha1Buf, 0x00, JBM_SHA1_BUF_SIZE);
    }

    ~JbmCommon()
    {
    }

public:
    int                 logLevel;

    static JbmCommon &instance();
    static JbmCommon *instancePtr();
    void setLogLevel(int level){ logLevel = level;}
    int log(int level, char *srcfile, int line, char *fmt, ...);
    int lfatal(char *srcfile, int line, char *fmt, ...);
    int lerror(char *srcfile, int line, char *fmt, ...);
    int lwarning(char *srcfile, int line, char *fmt, ...);
    int lnotice(char *srcfile, int line, char *fmt, ...);
    int ldebug(char *srcfile, int line, char *fmt, ...);

    const char *md5Hash(const unsigned char *str, unsigned int len);
    const char *md5Hash(const char *str, unsigned int len);
    const char *sha1Hash(const unsigned char *str, unsigned int len);
    const char *sha1Hash(const char *str, unsigned int len);
};




#endif // #ifndef __JBMCOMMON_H__
