#ifndef __JBMDATASET_h__
#define __JBMDATASET_h__
#include <algorithm>
#include <vector>
#include <map>
#include <string.h>

#define JBM_COLUMN  map<char *, unsigned int, JbmChrLt>
#define JBM_VALUE   vector<char *>
#define JBM_TYPE    vector<int>

using namespace std;

struct JbmChrLt
{
    bool operator()(char* s1, char* s2) const
    {
        return strcmp(s1, s2) < 0;
    }
};

class JbmRow
{
private:
    JBM_COLUMN      col;
    JBM_VALUE       var;
    JBM_TYPE        type;
public:
    bool insert(char *column, char *value, int tp)
    {
        JBM_COLUMN::iterator idx = col.find(column);

        if(idx != col.end())
        {
           var[(*idx).second] = value;
           type[(*idx).second] = tp;
        }else
        {
            col[column] = (unsigned int)var.size();
            var.push_back(value);
            type.push_back(tp);
        }
        return true;
    }

    const char *get(char *column)
    {
        JBM_COLUMN::iterator idx = col.find(column);

        if(idx != col.end())
            return (const char *)var[(*idx).second];
        else
            return NULL;
    }

    const char *get(unsigned int idx)
    {
        if(idx < var.size())
            return (const char *)var[idx];
        else
            return NULL;
    }

    unsigned long long getULL(char *column)
    {
        JBM_COLUMN::iterator idx = col.find(column);

        if(idx != col.end())
            return strtoll((const char *)var[(*idx).second], NULL, 10);
        else
            return NULL;
    }


    long long getLL(char *column)
    {
        JBM_COLUMN::iterator idx = col.find(column);

        if(idx != col.end())
            return strtoll((const char *)var[(*idx).second], NULL, 10);
        else
            return NULL;
    }

    unsigned int getUI(char *column)
    {
        JBM_COLUMN::iterator idx = col.find(column);

        if(idx != col.end())
            return (unsigned int)strtoul((const char *)var[(*idx).second], NULL, 10);
        else
            return NULL;
    }

    int getI(char *column)
    {
        JBM_COLUMN::iterator idx = col.find(column);

        if(idx != col.end())
            return atoi((const char *)var[(*idx).second]);
        else
            return NULL;
    }

    unsigned int cols()
    {
        return var.size();
    }

    unsigned int size()
    {
        return var.size();
    }

    ~JbmRow()
    {
    }

};

class JbmResult : public vector<JbmRow *>
{
public:
    ~JbmResult()
    {
        for(unsigned int i = 0; i < this->size(); i++)
        {
            delete (*this)[i];
        }
    }
};

class JbmTagsFinder
{
    char *_needle;
public:
    JbmTagsFinder(const char *needle)
    {
        _needle = (char *)calloc(1, strlen(needle)+1);
        strcpy(_needle, needle);
    }

    ~JbmTagsFinder()
    {
        free(_needle);
    }

    bool operator()(char *a) const
    {
        return strcmp(a, _needle)==0?true:false;
    }
};


class JbmTags : public vector<char *>
{
public:
    bool insert(char *tag)
    {
        if(find_if(this->begin(), this->end(), JbmTagsFinder(tag)) != this->end())
        {
            return false;
        }

        this->push_back(tag);
        return true;
    }
};

#endif //__JBMDATASET_h__
