#include "JbmMysql.h"

// {{{ JbmMysql::JbmMysql(const char *host, const char *user, const char *pass, const char *dbname, const int port = 3306)
JbmMysql::JbmMysql(const char *host, const char *user, const char *pass, const char *dbname, const int port = 3306)
{
    _pcon   = NULL;
    _affected_rows = 0;

    _host   = (char *)calloc(1, strlen(host)+1);
    strcpy(_host, host);
    _user   = (char *)calloc(1, strlen(user)+1);
    strcpy(_user, user);
    _pass   = (char *)calloc(1, strlen(pass)+1);
    strcpy(_pass, pass);
    _dbname = (char *)calloc(1, strlen(dbname)+1);
    strcpy(_dbname, dbname);
    _port   = port;
}
// }}}

// {{{ JbmMysql::~JbmMysql()
JbmMysql::~JbmMysql()
{
    if(_pcon == NULL) return;
    mysql_close(_pcon);
    free(_host);
    free(_user);
    free(_pass);
    free(_dbname);
}
// }}}

// {{{ bool JbmMysql::connect()
bool JbmMysql::connect()
{
    if(_pcon != NULL) return true;

    mysql_init(&_con);
    _pcon = mysql_real_connect(&_con,
                               _host,
                               _user,
                               _pass,
                               _dbname,
                               _port,
                               (char *)NULL,
                               0);
    if(_pcon == NULL || mysql_errno(_pcon) != 0) return false;

    return true;
}
// }}}

// {{{ const char *JbmMysql::error()
const char *JbmMysql::error()
{
    return mysql_error(_pcon);
}
// }}}

// {{{ int JbmMysql::errno()
int JbmMysql::errno()
{
    return mysql_errno(_pcon);
}
// }}}

// {{{ JbmResult JbmMysql::getRows(const char *query) // throw(const char *error_messag)
JbmResult JbmMysql::getRows(const char *query) // throw(const char *error_messag)
{
    _affected_rows = 0;
    if(!connect()) throw mysql_error(_pcon);

    JbmResult       result;
    JbmRow          *row;

    MYSQL_ROW       m_row;
    MYSQL_RES       *m_result;
    MYSQL_FIELD     *m_fields;
    int             m_qres, m_num_fields, cnt;
    unsigned long   *m_lengths;

    char            *col_name, *col_var;

    m_qres  = mysql_query(_pcon, query);
    if(m_qres != 0) throw mysql_error(_pcon);
    _affected_rows = mysql_affected_rows(_pcon);

    m_result = mysql_store_result(_pcon);

    m_num_fields  = mysql_num_fields(m_result);
    m_fields      = mysql_fetch_fields(m_result);

    while((m_row = mysql_fetch_row(m_result)) != NULL)
    {
        m_lengths = mysql_fetch_lengths(m_result);
        row = new JbmRow;

        for(cnt=0; cnt < m_num_fields; cnt++)
        {
            BB.set(m_fields[cnt].name, &col_name);
            BB.set(m_row[cnt], &col_var, m_lengths[cnt]);
            row->insert(col_name, col_var, m_fields[cnt].type);
        }
        result.push_back(row);
    }
    mysql_free_result(m_result);

    return result;
}
// }}}

// {{{ JbmRow JbmMysql::getRow(const char *query) // throw(const char *error_messag)
JbmRow JbmMysql::getRow(const char *query) // throw(const char *error_messag)
{
    _affected_rows = 0;
    if(!connect()) throw mysql_error(_pcon);

    JbmRow          row;

    MYSQL_ROW       m_row;
    MYSQL_RES       *m_result;
    MYSQL_FIELD     *m_fields;
    int             m_qres, m_num_fields, cnt;
    unsigned long   *m_lengths;

    char            *col_name, *col_var;

    m_qres  = mysql_query(_pcon, query);
    if(m_qres != 0) throw mysql_error(_pcon);
    _affected_rows = mysql_affected_rows(_pcon);

    m_result = mysql_store_result(_pcon);

    m_num_fields  = mysql_num_fields(m_result);
    m_fields      = mysql_fetch_fields(m_result);

    m_row = mysql_fetch_row(m_result);
    m_lengths = mysql_fetch_lengths(m_result);
    if(m_row == NULL) throw "Not found matched row.";

    for(cnt=0; cnt < m_num_fields; cnt++)
    {
        BB.set(m_fields[cnt].name, &col_name);
        BB.set(m_row[cnt], &col_var, m_lengths[cnt]);
        row.insert(col_name, col_var, m_fields[cnt].type);
    }
    mysql_free_result(m_result);
    return row;
}
// }}}

// {{{ const char *JbmMysql::getOne(const char *query) // throw(const char *error_messag)
const char *JbmMysql::getOne(const char *query) // throw(const char *error_messag)
{
    _affected_rows = 0;
    if(!connect()) throw mysql_error(_pcon);

    MYSQL_ROW       m_row;
    MYSQL_RES       *m_result;
    int             m_qres, m_num_fields;
    unsigned long   *m_lengths;

    char            *col_var;

    m_qres          = mysql_query(_pcon, query);
    if(m_qres != 0) throw mysql_error(_pcon);

    _affected_rows  = mysql_affected_rows(_pcon);
    if(_affected_rows == 0) throw "Not found matched row.";

    m_result        = mysql_store_result(_pcon);
    m_num_fields    = mysql_num_fields(m_result);
    m_row           = mysql_fetch_row(m_result);
    if(m_row == NULL) throw "Not found matched row.";

    m_lengths       = mysql_fetch_lengths(m_result);
    BB.set(m_row[0], &col_var, m_lengths[0]);

    mysql_free_result(m_result);
    return (const char *)col_var;
}
// }}}

// {{{ bool JbmMysql::query(const char *query) // throw(const char *error_messag)
bool JbmMysql::query(const char *query) // throw(const char *error_messag)
{
    _affected_rows = 0;
    if(!connect()) throw mysql_error(_pcon);

    int             m_qres;

    m_qres  = mysql_query(_pcon, query);
    if(m_qres != 0 || mysql_errno(_pcon) != 0) throw mysql_error(_pcon);
    _affected_rows = mysql_affected_rows(_pcon);

    return true;
}
// }}}

// {{{ int JbmMysql::affected_rows()
int JbmMysql::affected_rows()
{
    return _affected_rows;
}
// }}}
