#include "JbmProc.h"

//
// user function.
//
// {{{ JbmProc::addUser(const char *user_id, const char *user_pass, const char *email)
bool JbmProc::addUser(const char *user_id, const char *user_pass, const char *email)
{
    query  = "INSERT INTO bookmark_user \n"
             "VALUES (\n"
             "        '"JBM_JOINQS(user_id)"',\n"
             "        '"JBM_JOINQS(JC.md5Hash(user_pass, strlen(user_pass)))"',\n"
             "        '"JBM_JOINQS(email)"',\n"
             "        now()\n"
             "       )\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        db.query(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    
    return true;
}
// }}}


// {{{ bool JbmProc::deleteUser(const char *user_id)
bool JbmProc::deleteUser(const char *user_id)
{
    query  = "DELETE FROM bookmark_user \n"
             " WHERE bu_user_id = '"JBM_JOINQS(user_id)"'\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        db.query(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    
    return true;
}
// }}}



//
// auth_key : md5(user_id + md5(user_pass) + user_ip)
//
// {{{ bool JbmProc::checkLogin(const char *user_id, const char *auth_key, const char *user_ip)
bool JbmProc::checkLogin(const char *user_id, const char *auth_key, const char *user_ip)
{
    query = "SELECT * \n"
            "  FROM bookmark_user\n"
            " WHERE bu_user_id = '"JBM_JOINQS(user_id)"'\n"
            "   AND MD5(CONCAT(bu_user_id, bu_password, '"JBM_JOINQS(user_ip)"')) = '"JBM_JOINQS(auth_key)"'\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        JbmRow row = db.getRow(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        if(db.affected_rows() == 0)
        {
            JC.lnotice(__FILE__, __LINE__, "Not found user. affected_rows = 0\n");
            return false;
        }
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    
    return true;
}
// }}}


// α Ҷ 
// Args 
//  user_id     :
//  login_key   : md5(user_id + md5(user_pass))
//  user_ip     : 
//
//  auth_key    :  args  α Ͽٸ
//                  auth_key ä.
// {{{ bool JbmProc::login(const char *user_id, const char *login_key, const char *user_ip, char **auth_key)
bool JbmProc::login(const char *user_id, const char *login_key, const char *user_ip, char **auth_key)
{
    query = "SELECT MD5(concat(bu_user_id, bu_password, '"JBM_JOINQS(user_ip)"')) auth_key   \n"
            "  FROM bookmark_user\n"
            " WHERE bu_user_id = '"JBM_JOINQS(user_id)"'\n"
            "   AND md5(concat(bu_user_id, bu_password)) = '"JBM_JOINQS(login_key)"' \n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        JbmRow row = db.getRow(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        if(db.affected_rows() == 0)
        {
            JC.lnotice(__FILE__, __LINE__, "Not found user. affected_rows = 0\n");
            return false;
        }
        JC.ldebug(__FILE__, __LINE__, "Login UserId:%s, LoginKey:%s, UserIp:%s, AuthKey:%s\n",
                user_id, login_key, user_ip, row.get("auth_key"));
        *auth_key = (char *)row.get("auth_key");
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    
    return true;
}
// }}}


//
// Item function.
//
// {{{ JbmRow JbmProc::getItem(const char *user_id, unsigned long long item_key)
JbmRow JbmProc::getItem(const char *user_id, unsigned long long item_key)
{
    query = "SELECT *\n"
            "  FROM bookmark_item\n"
            " WHERE bi_user_id    = '"JBM_JOINQS(user_id)"'\n"
            "   AND bi_item_key   = '"JBM_JOINQLL(item_key)"' \n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());

    JbmRow row;
    try
    {
        row = db.getRow(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        if(db.affected_rows() == 0)
        {
            JC.lnotice(__FILE__, __LINE__, "Not found user. affected_rows = 0\n");
        }
        JC.ldebug(__FILE__, __LINE__, "UserId:%s, parentKey:%lld\n",
                user_id, item_key);

        return row;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return row;
    }
    
    return row;
}
// }}}


// {{{ JbmResult JbmProc::getItemsByParentKey(const char *user_id, unsigned long long parent_key, bool public_only = false)
JbmResult JbmProc::getItemsByParentKey(const char *user_id, unsigned long long parent_key, bool public_only = false)
{
    query = "SELECT *\n"
            "  FROM bookmark_item\n"
            " WHERE bi_user_id    = '"JBM_JOINQS(user_id)"'\n"
            "   AND bi_parent_key = '"JBM_JOINQLL(parent_key)"' \n";
    if(public_only)
    {
        query += "   AND bi_is_public = 'Y'";
    }

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());

    JbmResult result;
    try
    {
        result = db.getRows(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        if(db.affected_rows() == 0)
        {
            JC.lnotice(__FILE__, __LINE__, "Not found user. affected_rows = 0\n");
        }
        JC.ldebug(__FILE__, __LINE__, "UserId:%s, parentKey:%lld, publicOnly:%s\n",
                user_id, parent_key, (public_only?"Y":"N"));

        return result;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return result;
    }
    
    return result;
}
// }}}


// {{{ JbmResult JbmProc::getItemTags(const char *user_id, unsigned long long item_key)
JbmResult JbmProc::getItemTags(const char *user_id, unsigned long long item_key)
{
    query = "SELECT *\n"
            "  FROM bookmark_tags, bookmark_tag_link\n"
            " WHERE btl_user_id    = '"JBM_JOINQS(user_id)"'\n"
            "   AND btl_item_key   = '"JBM_JOINQLL(item_key)"'\n"
            "   AND btl_tag_key    = bt_tag_key\n"
            " ORDER BY btl_create_time DESC\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());

    JbmResult result;
    try
    {
        result = db.getRows(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");

        if(db.affected_rows() == 0)
        {
            JC.lnotice(__FILE__, __LINE__, "Not found user. affected_rows = 0\n");
        }

        JC.ldebug(__FILE__, __LINE__, "ItemKey:%lld\n", item_key);

        return result;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return result;
    }
    
    return result;
}
// }}}


// {{{ JbmResult JbmProc::getItemByTagKey(unsigned long long tag_key)
JbmResult JbmProc::getItemByTagKey(unsigned long long tag_key)
{
    query = "SELECT *\n"
            "  FROM bookmark_item, bookmark_tag_link\n"
            " WHERE btl_tag_key    = '"JBM_JOINQLL(tag_key)"'\n"
            "   AND btl_user_id    = bi_user_id\n"
            "   AND btl_item_key   = bi_item_key\n"
            " ORDER BY btl_create_time DESC\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());

    JbmResult result;
    try
    {
        result = db.getRows(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        if(db.affected_rows() == 0)
        {
            JC.lnotice(__FILE__, __LINE__, "Not found user. affected_rows = 0\n");
        }
        JC.ldebug(__FILE__, __LINE__, "TagKey:%lld\n",
                tag_key);

        return result;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return result;
    }
    
    return result;
}
// }}}


// {{{ bool JbmProc::updateItemTitle(const char *user_id, unsigned long long item_key, const char *title)
bool JbmProc::updateItemTitle(const char *user_id, unsigned long long item_key, const char *title)
{
    query  = "UPDATE bookmark_item \n"
             "   SET bi_title = '"JBM_JOINQS(title)"'\n"
             " WHERE bi_user_id  = '"JBM_JOINQS(user_id)"'\n"
             "   AND bi_item_key = '"JBM_JOINQLL(item_key)"'\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        db.query(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    return true;
}
// }}}


// {{{ bool JbmProc::updateItemLink(const char *user_id, unsigned long long item_key, const char *link)
bool JbmProc::updateItemLink(const char *user_id, unsigned long long item_key, const char *link)
{
    query  = "UPDATE bookmark_item \n"
             "   SET bi_link     = '"JBM_JOINQS(link)"'\n"
             " WHERE bi_user_id  = '"JBM_JOINQS(user_id)"'\n"
             "   AND bi_item_key = '"JBM_JOINQLL(item_key)"'\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        db.query(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    return true;
}
// }}}


// {{{ bool JbmProc::updateItemPublic(const char *user_id, unsigned long long item_key, bool is_public)
bool JbmProc::updateItemPublic(const char *user_id, unsigned long long item_key, bool is_public)
{
    JbmRow row = getItem(user_id, item_key);
    if(row.size() == 0)
    {
        JC.lnotice(__FILE__, __LINE__, "Not found item. user_id:%s, item_key:%s\n", user_id, item_key);
        return false;
    }
    if(JBM_YN2TF(row.get("bi_is_public")) == is_public)
    {
        JC.lnotice(__FILE__, __LINE__, "No need change. user_id:%s, item_key:%s, ori_public:%s, after_public:%s\n",
                user_id, item_key, row.get("bi_is_public"), (is_public?"Y":"N"));
        return false;
    }

    query  = "UPDATE bookmark_item \n"
             "   SET bi_is_public = '"JBM_JOINQTF(is_public)"'\n"
             " WHERE bi_user_id   = '"JBM_JOINQS(user_id)"'\n"
             "   AND bi_item_key  = '"JBM_JOINQLL(item_key)"'\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        db.query(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");

        JbmResult tags = getItemTags(user_id, item_key);
        for(unsigned int i=0; i<tags.size();i++)
        {
            chagneTagCount(JBM_STR2ULL(tags[i]->get("bt_tag_key")), is_public);
        }
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    return true;
}
// }}}

// {{{ bool JbmProc::updateItemParent(const char *user_id, unsigned long long item_key, unsigned long long parent_key)
bool JbmProc::updateItemParent(const char *user_id, unsigned long long item_key, unsigned long long parent_key)
{
    JbmRow row = getItem(user_id, parent_key);
    if(row.size() == 0)
    {
        JC.lnotice(__FILE__, __LINE__, "Parent is not exists. user_id:%s, item_key:%lld, parent_key:%lld\n",
                user_id, item_key, parent_key);
        return false;
    }
    if(JBM_YN2TF(row.get("bi_is_folder")) == false)
    {
        JC.lnotice(__FILE__, __LINE__, "Parent is not folder.user_id:%s, item_key:%lld, parent_key:%lld\n",
                user_id, item_key, parent_key);
        return false;
    }

    return true;
}
// }}}


//
// Parameter in row('M'andatory, 'O'ptional)
//   user_id:M
//   item_key:M
//   title:M
//   parent_key:O       : Parent item_key. If it set '0', root item. default '0'
//   is_public:O        : Public setting. default 'N'
//   desc               : Description. default ''
// {{{ bool JbmProc::addItemFolder(JbmRow &row, JbmTags &tags)
// Parameter in row('M'andatory, 'O'ptional)
//   tag_name:M
bool JbmProc::addItemFolder(JbmRow &row, JbmTags &tags)
{
    if(row.get("user_id") == NULL ||
       row.get("item_key") == NULL ||
       row.get("title") == NULL)
    {
        JC.lerror(__FILE__, __LINE__, "Not enought mandatory value(s). user_id:%s, item_key:%s, title:%s\n",
                row.get("user_id"), row.get("item_key"), row.get("title"));
        return false;
    }

    unsigned long long item_key = getItemLastKey(row.get("user_id"));
    unsigned long long parent_key = 0;

    query = "INSERT INTO bookmark_item\n"
            "VALUES (\n"
            "        '"JBM_JOINQS(row.get("user_id"))"',\n"
            "        '"JBM_JOINQLL(item_key)"',\n";
    if(row.get("parent_key") == NULL)
    {
        query += "        '0'\n";
    }else
    {
        parent_key = JBM_STR2ULL(row.get("parent_key"));
        query += "        '"JBM_JOINQS(row.get("parent_key"))"'\n";
    }

    unsigned int order = getItemLastOrder(row.get("user_id"), parent_key);
    bool is_public     = false;

    query +="        '"JBM_JOINQI(order)"',\n";

    if(row.get("is_public") == NULL)
    {
        query += "        'N'\n";
    }else
    {
        is_public       = JBM_YN2TF(row.get("is_public"));
        query += "        '"JBM_JOINQS(row.get("is_public"))"'\n";
    }

    query +="        'Y',\n"
            "        now(),\n"
            "        now(),\n"
            "        '0',\n"
            "        '"JBM_JOINQS(row.get("title"))"',\n"
            "        '"JBM_JOINQI(tags.size())"',\n"
            "        '',\n";

    if(row.get("desc") == NULL)
        query += "        '',\n";
    else
        query += "        '"JBM_JOINQS(row.get("desc"))"'\n";

    query +="       )\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        db.query(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");

        for(unsigned int i=0; i<tags.size();i++)
        {
			JbmRow tagRow = getTag(tags[i]);
			if(tagRow.size() == 0)
			{
				addTag(tagRow);
				tagRow = getTag(tags[i]);
				sdkfjalsjdfjsdklfjalkj
			}

        }
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    return true;
}
// }}}


// {{{ unsigned int JbmProc::getItemLastKey(const char *user_id)
unsigned int JbmProc::getItemLastKey(const char *user_id)
{
    query  = "SELECT max(bi_item_key) max\n"
             "  FROM bookmark_item\n"
             " WHERE bi_user_id   = '"JBM_JOINQS(user_id)"'\n"
             " GROUP BY bi_user_id\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());

    try
    {
        return atoi(db.getOne(query.c_str()));
    }catch(const char *err)
    {
        if(db.errno() == 0) return 0;
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return 0;
    }
}
// }}}


// {{{ unsigned long long JbmProc::getItemLastKey(const char *user_id, unsigned long long parent_key)
unsigned long long JbmProc::getItemLastOrder(const char *user_id, unsigned long long parent_key)
{
    query  = "SELECT max(bi_order) max\n"
             "  FROM bookmark_item\n"
             " WHERE bi_user_id    = '"JBM_JOINQS(user_id)"'\n"
             " WHERE bi_parent_key = '"JBM_JOINQLL(parent_key)"'\n"
             " GROUP BY bi_user_id, bi_parent_key\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());

    try
    {
        return atoi(db.getOne(query.c_str()));
    }catch(const char *err)
    {
        if(db.errno() == 0) return 0;
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return 0;
    }
}
// }}}


// {{{ bool JbmProc::addItemLink(JbmRow &row)
bool JbmProc::addItemLink(JbmRow &row)
{
    return true;
}
// }}}


// {{{ bool JbmProc::deleteItemLink(const char *user_id, unsigned long long item_key)
bool JbmProc::deleteItemLink(const char *user_id, unsigned long long item_key)
{
    return true;
}
// }}}


// {{{ JbmResult JbmProc::searchTags(const char *keyword)
JbmResult JbmProc::searchTags(const char *keyword)
{
    return JbmResult();
}
// }}}


// {{{ JbmRow JbmProc::getTag(const char *tag_name)
JbmRow JbmProc::getTag(const char *tag_name)
{
	const char *lower = changeTagNameLower(tag_name);
			
    query = "SELECT *\n"
            "  FROM bookmark_tags\n"
            " WHERE bt_tag_name_lower = '"JBM_JOINQS(lower)"'\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());

    JbmRow row;
    try
    {
        row = db.getRow(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        if(db.affected_rows() == 0)
        {
            JC.lnotice(__FILE__, __LINE__, "Not found user. affected_rows = 0\n");
        }
        JC.ldebug(__FILE__, __LINE__, "tagNameLower:%s\n", lower);

        return row;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return row;
    }
    
    return row;
}
// }}}


// {{{ JbmRow JbmProc::getTag(unsigned long long tag_key)
JbmRow JbmProc::getTag(unsigned long long tag_key)
{
    query = "SELECT *\n"
            "  FROM bookmark_tags\n"
            " WHERE bt_tag_key = '"JBM_JOINQLL(tag_key)"'\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());

    JbmRow row;
    try
    {
        row = db.getRow(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        if(db.affected_rows() == 0)
        {
            JC.lnotice(__FILE__, __LINE__, "Not found user. affected_rows = 0\n");
        }
        JC.ldebug(__FILE__, __LINE__, "tag_key:%lld\n", tag_key);

        return row;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return row;
    }
    
    return row;
}
// }}}


// {{{ unsigned int JbmProc::getTagLinkCount(unsigned long long tag_key)
unsigned int JbmProc::getTagLinkCount(unsigned long long tag_key)
{
    return 0;
}
// }}}


// {{{ unsigned int JbmProc::getTagLinkCountPublic(unsigned long long tag_key)
unsigned int JbmProc::getTagLinkCountPublic(unsigned long long tag_key)
{
    return 0;
}
// }}}


//
// Parameter in row('M'andatory, 'O'ptional)
//   tag_name:M
//   creator:M
// {{{ bool JbmProc::addTag(JbmRow &row)
bool JbmProc::addTag(JbmRow &row)
{
	if(row.get("tag_name") == NULL || row.get("creator") == NULL)
	{
        JC.lerror(__FILE__, __LINE__, "Not enought mandatory value(s). tag_name:%s, creator:%s",
                row.get("tag_name"), row.get("creator"));
        return false;
    }

	const char *tag_name_lower = changeTagNameLower(row.get("tag_name"));

    query = "INSERT INTO bookmark_tags\n"
            "VALUES (\n"
            "        '',\n"
			"        '"JBM_JOINQS(tag_name_lower)"',\n"
			"        '"JBM_JOINQS(row.get("tab_name"))"',\n"
            "        '0',\n"
            "        '0',\n"
			"        '"JBM_JOINQS(row.get("creator"))"',\n"
            "        now(),\n"
            "        now(),\n"
            "       )\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());

    try
    {
        db.query(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    return true;
}
// }}}

// {{{ bool JbmProc::changeTagNameLower(const char *tag_name)
const char *JbmProc::changeTagNameLower(const char *tag_name)
{
	char *tag_name_lower;
	BB.set(tag_name, &tag_name_lower);

	unsigned char *lowerIdx, *lowerEnd;

	lowerIdx = (unsigned char *)tag_name_lower;
	lowerEnd = lowerIdx + strlen(tag_name_lower);

	while(lowerIdx < lowerEnd)
	{
		*lowerIdx = tolower(*lowerIdx);
		lowerIdx++;
	}
			
	return tag_name_lower;
}
// }}}


// {{{ bool JbmProc::increaseTagCount(unsigned long long tag_key, bool is_public)
bool JbmProc::increaseTagCount(unsigned long long tag_key, bool is_public)
{
    if(is_public)
    {
        query  = "UPDATE bookmark_tags \n"
                 "   SET bt_count        = bt_count + 1,\n"
                 "       bt_count_public = bt_count_public + 1\n"
                 " WHERE bt_tag_key      = '"JBM_JOINQLL(tag_key)"'\n";
    }else
    {
        query  = "UPDATE bookmark_tags \n"
                 "   SET bt_count        = bt_count + 1\n"
                 " WHERE bt_tag_key      = '"JBM_JOINQLL(tag_key)"'\n";
    }

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        db.query(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    return true;
}
// }}}


// {{{ bool JbmProc::decreaseTagCount(unsigned long long tag_key, bool is_public)
bool JbmProc::decreaseTagCount(unsigned long long tag_key, bool is_public)
{
    if(is_public)
    {
        query  = "UPDATE bookmark_tags \n"
                 "   SET bt_count        = bt_count - 1,\n"
                 "       bt_count_public = bt_count_public - 1\n"
                 " WHERE bt_tag_key      = '"JBM_JOINQLL(tag_key)"'\n";
    }else
    {
        query  = "UPDATE bookmark_tags \n"
                 "   SET bt_count        = bt_count - 1\n"
                 " WHERE bt_tag_key      = '"JBM_JOINQLL(tag_key)"'\n";
    }

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        db.query(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    return true;
}
// }}}


// {{{ bool JbmProc::chagneTagCount(unsigned long long tag_key, bool is_public)
bool JbmProc::chagneTagCount(unsigned long long tag_key, bool is_public)
{
    if(is_public)
    {
        query  = "UPDATE bookmark_tags \n"
                 "   SET bt_count_public = bt_count_public + 1\n"
                 " WHERE bt_tag_key      = '"JBM_JOINQLL(tag_key)"'\n";
    }else
    {
        query  = "UPDATE bookmark_tags \n"
                 "   SET bt_count_public = bt_count_public - 1\n"
                 " WHERE bt_tag_key      = '"JBM_JOINQLL(tag_key)"'\n";
    }

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        db.query(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    return true;
}
// }}}


// {{{ bool JbmProc::arrangeTagCount(unsigned long long tag_key)
bool JbmProc::arrangeTagCount(unsigned long long tag_key)
{
    unsigned int all = 0, pub = 0;
    all = countTagAll(tag_key);
    pub = countTagPublic(tag_key);

    query  = "UPDATE bookmark_tags \n"
             "   SET bt_count        = '"JBM_JOINQI(all)"'\n"
             "   SET bt_count_public = '"JBM_JOINQI(pub)"'\n"
             " WHERE bt_tag_key      = '"JBM_JOINQLL(tag_key)"'\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());
    try
    {
        db.query(query.c_str());
        JC.ldebug(__FILE__, __LINE__, "Query OK.\n");
        return true;
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return false;
    }
    return true;
}
// }}}


// {{{ unsigned int JbmProc::countTagAll(unsigned long long tag_key)
unsigned int JbmProc::countTagAll(unsigned long long tag_key)
{
    query  = "SELECT count(*)\n"
             "  FROM bookmark_tag_link \n"
             " WHERE btl_tag_key  = '"JBM_JOINQLL(tag_key)"'\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());

    try
    {
        return atoi(db.getOne(query.c_str()));
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return 0;
    }
}
// }}}


// {{{ unsigned int JbmProc::countTagPublic(unsigned long long tag_key)
unsigned int JbmProc::countTagPublic(unsigned long long tag_key)
{
    query  = "SELECT count(*) cnt\n"
             "  FROM bookmark_tag_link, bookmark_item \n"
             " WHERE btl_tag_key  = '"JBM_JOINQLL(tag_key)"'\n"
             " WHERE btl_user_id  = bi_user_id\n"
             " WHERE btl_item_key = bi_item_key\n"
             " WHERE bi_is_public = 'Y'\n";

    JC.ldebug(__FILE__, __LINE__, "QUERY - %s \n", query.c_str());

    try
    {
        return atoi(db.getOne(query.c_str()));
    }catch(const char *err)
    {
        error = err;
        JC.lerror(__FILE__, __LINE__, "Query error - %s \n", err);
        return 0;
    }
}
// }}}


// {{{ bool JbmProc::deleteTagLink(const char *user_id, unsigned long long item_key, unsigned long long tag_key)
bool JbmProc::deleteTagLink(const char *user_id, unsigned long long item_key, unsigned long long tag_key)
{
    return true;
}
// }}}

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */

