#ifndef __JBMPROC_H__
#define __JBMPROC_H__

#include "BulkBuffer.h"
#include "JbmMysql.h"
#include "JbmDataset.h"
#include <string>
#include <ctype.h>

#define JBM_YN2TF(x)        (strncmp(x,"Y",1)==0?true:false)
#define JBM_TF2YN(x)        (x?"Y":"N")
#define JBM_STR2ULL(x)      strtoull(x, NULL, 10)

#define JBM_JOINQS(x)       ;query+=x;query+=
#define JBM_JOINQLL(x)      ;sprintf(_queryConvertBuffer, "%lld", x); query+=_queryConvertBuffer;query+=
#define JBM_JOINQI(x)       ;sprintf(_queryConvertBuffer, "%d", x); query+=_queryConvertBuffer;query+=
#define JBM_JOINQTF(x)      ;query+=(x?"Y":"N");query+=


#define JBM_QUERY_CONVERT_BUFFER_SIZE   1024

#define JBM_MYSQL_DEFAULT_PORT  3306

class JbmProc
{
private:
    JbmMysql    db;
    string      query;

    char    *_host, *_user, *_pass, *_dbname;
    int     _port;
    char    _queryConvertBuffer[JBM_QUERY_CONVERT_BUFFER_SIZE];

public:
    string      error;
    int         errno;

    JbmProc(const char *host,
            const char *user,
            const char *pass,
            const char *dbname,
            const int port = JBM_MYSQL_DEFAULT_PORT)
    :db(host, user, pass, dbname, port)
    {
    }

    bool addUser(const char *user_id, const char *auth_key, const char *email);
    bool deleteUser(const char *user_id);
    bool checkLogin(const char *user_id, const char *auth_key, const char *user_ip);
    bool login(const char *user_id, const char *login_key, const char *user_ip, char **auth_key);

    JbmRow getItem(const char *user_id, unsigned long long item_key);
    JbmResult getItemsByParentKey(const char *user_id, unsigned long long parent_key, bool public_only = false);
    JbmResult getItemTags(const char *user_id, unsigned long long item_key);
    JbmResult getItemByTagKey(unsigned long long tag_key);
    unsigned int getItemLastKey(const char *user_id);
    unsigned long long getItemLastOrder(const char *user_id, unsigned long long parent_key);

    bool updateItemTitle(const char *user_id, unsigned long long item_key, const char *title);
    bool updateItemLink(const char *user_id, unsigned long long item_key, const char *link);
    bool updateItemPublic(const char *user_id, unsigned long long item_key, bool is_public);
    bool updateItemParent(const char *user_id, unsigned long long item_key, unsigned long long parent_key);
    bool addItemFolder(JbmRow &row, JbmTags &tags);
    bool addItemLink(JbmRow &row);
    bool increaseTagCount(unsigned long long tag_key, bool is_public);
    bool decreaseTagCount(unsigned long long tag_key, bool is_public);
    bool chagneTagCount(unsigned long long tag_key, bool is_public);
    bool arrangeTagCount(unsigned long long tag_key);
    unsigned int countTagAll(unsigned long long tag_key);
    unsigned int countTagPublic(unsigned long long tag_key);

    bool deleteItemLink(const char *user_id, unsigned long long item_key);

    JbmResult searchTags(const char *keyword);
    JbmRow getTag(unsigned long long tag_key);
    JbmRow getTag(const char *tag_name);

    unsigned int getTagLinkCount(unsigned long long tag_key);
    unsigned int getTagLinkCountPublic(unsigned long long tag_key);

    bool addTag(JbmRow &row);

    const char *changeTagNameLower(const char *tag_name);

    bool deleteTagLink(const char *user_id, unsigned long long item_key, unsigned long long tag_key);
};

#endif //__JBMPROC_H__
